/* Example: National Lottery numbers selector */
/* An alternative version, based on a different data model */

#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#define BALLS 49  /* the total number of balls */
#define DRAWN 6   /* the number of balls drawn */

void choose_numbers(int selected[BALLS + 1]);
void print_numbers(int selected[BALLS + 1]);
int random_ball(int min, int max);
void seed_rand(void);

/************************************************/

int main(void)
{
  int selected[BALLS + 1];  /* Array of numbers selected. Index runs 
                            from 1 to BALLS, to prevent out-by-one 
                            errors. An entry selected[i] == 0 means 
                            number i is not selected, 1 means 
                            selected. */
  choose_numbers(selected);
  print_numbers(selected);
  
  return EXIT_SUCCESS;
}

/************************************************/

void choose_numbers(int selected[BALLS + 1])
{
  int i, n;
  
  seed_rand();  /* seed rand (<stdlib.h>) before use */
  
  for (i = 1; i <= BALLS; i++)  /* zero out the array */
    selected[i] = 0;
  
  /* Choose numbers, rejecting each if it has already been chosen: */

  for (i = 0; i < DRAWN; i++)
  {
    do
    {
      n = random_ball(1, BALLS);
    }
      while (selected[n]);
    selected[n] = 1;
  }

  return;
}

/************************************************/

void print_numbers(int selected[BALLS + 1])
{
  /* Print the selected numbers */
  /* Needs <stdio.h> */
  
  int i;
  
  puts("National Lottery number selector program\n");
  printf("Your lucky numbers are: ");
  
  for (i = 1; i <= BALLS; i++)
    if (selected[i])
      printf("%i ", i);
        
  puts("\nGood luck!");

  return;
}

/************************************************/

void seed_rand(void)
{
  /* Seed the random number generator from the system time */
  /* Needs <stdlib.h> and <time.h> */
  
  time_t t;  /* data type time_t is defined in <time.h> */
  
  time(&t);  /* time, in seconds, since the start of 1968 */
  srand(t);
  return;
}

/************************************************/

int random_ball(int min, int max)
{
  /* Returns a random integer from min to max */
  /* Needs <stdlib.h> */

  double x;
  
  x = (double) rand(); /* Function rand (in <stdlib.h>) returns 
                       an integer from 0 to RAND_MAX */
  
  x /= ((double) RAND_MAX + 1);
                       /* A double from 0 to just less than 1.
                       The cast avoids overflow problems. */
  
  x = x * (max - min + 1) + min;
                /* a double from min to just less than max + 1 */
  
  return (int) x;   /* an int from min to max */
}

/************************************************/
